/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef UI_UTILS_H_
#define UI_UTILS_H_

#include <Evas.h>
#include <camera.h>
#include <recorder.h>
#include <app_common.h>

/**
 * @brief Add naviframe with default settings.
 * @param[in] parent   Evas_Object parent
 * @return new naviframe object
 */
Evas_Object *ui_utils_navi_add(Evas_Object *parent);

/**
 * @brief Add default layout.
 * @param[in] parent   Evas_Object parent
 * @param[in] destroy_cb   destroy callback
 * @param[in] cb_data   user data
 * @return new layout object
 */
Evas_Object *ui_utils_layout_add(Evas_Object *parent, Evas_Object_Event_Cb destroy_cb, void *cb_data);

/**
 * @brief Adjusts recorder orientation to camera orientation
 * @param[in] recorder_h recorder recorder-handle
 * @param[in] camera_h camera camera-handle
 * @param[in] app_device_orientation_e current device orientation
 */
void ui_utils_set_camera_orientation(recorder_h recorder, camera_h camera,
		app_device_orientation_e current_rotation);
#endif /* UI_UTILS_H_ */
