/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "../inc/ui-utils.h"

#include <app.h>
#include <Elementary.h>

Evas_Object *ui_utils_navi_add(Evas_Object *parent)
{
    if(!parent)
    {
        return NULL;
    }
    Evas_Object *navi = elm_naviframe_add(parent);
    elm_naviframe_prev_btn_auto_pushed_set(navi, EINA_FALSE);

    return navi;
}

Evas_Object *ui_utils_layout_add(Evas_Object *parent, Evas_Object_Event_Cb destroy_cb, void *cb_data)
{
    Evas_Object * layout = elm_layout_add(parent);
    if(!layout)
    {
        return NULL;
    }

    elm_layout_theme_set(layout, "layout", "application", "default");
    evas_object_event_callback_add(layout, EVAS_CALLBACK_FREE, destroy_cb, cb_data);

    return layout;
}

void ui_utils_set_camera_orientation(recorder_h recorder, camera_h camera,
		app_device_orientation_e current_rotation)
{
    recorder_rotation_e rotation;
    int lens = 0;
    int video_rotation = 0;
    int display_rotation = 0;

    camera_attr_get_lens_orientation(camera, &lens);
    switch(current_rotation)
    {
        case APP_DEVICE_ORIENTATION_0:
           display_rotation = APP_DEVICE_ORIENTATION_0;
           break;
        case APP_DEVICE_ORIENTATION_90:
           display_rotation = APP_DEVICE_ORIENTATION_270;
           break;
        case APP_DEVICE_ORIENTATION_180:
           display_rotation = APP_DEVICE_ORIENTATION_180;
           break;
        case APP_DEVICE_ORIENTATION_270:
           display_rotation = APP_DEVICE_ORIENTATION_90;
           break;
        default:
           display_rotation = RECORDER_ROTATION_NONE;
           break;
     }

    video_rotation = (360 + lens - display_rotation) % 360;
    switch(video_rotation)  {
        case APP_DEVICE_ORIENTATION_0:
           rotation = RECORDER_ROTATION_NONE;
           break;
        case APP_DEVICE_ORIENTATION_90:
           rotation = RECORDER_ROTATION_90;
           break;
        case APP_DEVICE_ORIENTATION_180:
           rotation = RECORDER_ROTATION_180;
           break;
        case APP_DEVICE_ORIENTATION_270:
           rotation = RECORDER_ROTATION_270;
           break;
        default:
           rotation = RECORDER_ROTATION_NONE;
           break;
    }

    recorder_attr_set_orientation_tag(recorder, rotation);
}

